#!/bin/sh
# the next line restarts using rwish \
exec elwish "$0" "$@"
#
# Copyright (c) 2000-2007 LAAS/CNRS 
#
# Matthieu Herrb - Mon Aug 28 2000
#
# $Id: seglocEdit.tcl,v 1.17 2007/10/15 10:47:45 sara Exp $
#

# mkIndex mode
if {![info exists argv0]} return

namespace eval Grh2 {
}

# find out base directory of g2h2
set dir [file join [ file dirname $argv0 ] .. ]
set ::Grh2::dir $dir

# load BWidget
lappend auto_path $dir/tcl/BWidget
package require BWidget

# load various library functions
source $dir/tcl/canvas.tcl
source $dir/tcl/coord.tcl
source $dir/tcl/tk_entrydialog.tcl


# Create the ::Grh2::segloc namespace
namespace eval ::Grh2::seg {
#    variable segments
    variable currentFileName
    variable currentFileNum
    variable mousePosition
    variable robAngle
    variable robX
    variable robY
    variable posByPosterFlag
    variable loopFlag
    variable xrect1
    variable yrect1
    variable xrect2
    variable yrect2
}

# Default colors for the possible states of a segment

set stateColor(normal) black
set stateColor(current) lightblue
set stateColor(selected) blue
set stateColor(laser) green

set Grh2::seg::mousePosition "x 0.0 y 0.0"
set Grh2::seg::robotPosition "rob: 0.00 m 0.00 m 0.000 rd"
set Grh2::seg::rqst "No request on"
set Grh2::seg::nbsegs "0 segs"
set Grh2::seg::posByPosterFlag 1
set Grh2::seg::robX 0
set Grh2::seg::robY 0
set Grh2::seg::robAngle 0
set Grh2::seg::loopFlag 1
set Grh2::seg::xrect1 0
set Grh2::seg::xrect2 0

#----------------------------------------------------------------------
#
# Set up main window
#
proc Grh2::mainWindow { } {
    global grh2

    set descmenu {
	"&File" all file 0 {
	    {command "&Init module..." {} "Init segloc module (but with no connections)" {} \
		 -command Grh2::seg::initModule}
	    {command "&Load map..." {} "Load map" {} \
		 -command Grh2::seg::loadMap}
	    {command "Localize robot" {} "Localize the robot (first place it approximately using the mouse, turn it with <CTRL>-mouse)" {} \
		 -command Grh2::seg::localize}
	    {command "&Save map" {} "Save current map" {} \
		 -command Grh2::seg::saveMap}
	    {command "Save &As" {} "Save current map in a new file" {} \
		 -command Grh2::seg::saveMapAs}
	    {command "E&xit" {} "Exit Grh2" {} -command Grh2::exit}
	}
	"&Edit" all edit 0 {
	    {command "&Delete selected segments" {} "Delete selected segment(s)" {} \
		 -command Grh2::seg::deleteSelected}
	    {command "Clip in rectangle" {} "Delete segments in rectangle (first make rectangle using the mouse and pressing <SHIFT>-<BUTTON1>)" {} \
		 -command "Grh2::seg::clip SEGLOC_REMOVE_IN"}
	    {command "Clip out rectangle" {} "Delete segments out rectangle (first make rectangle using the mouse and pressing: <SHIFT>-<BUTTON1>)" {} \
		 -command "Grh2::seg::clip SEGLOC_REMOVE_OUT"}
	    {command "&Merge segments" {} "Merge 2 selected segments" {} \
		 -command Grh2::seg::merge}
	    {command "&Intersect segments" {} "Intersect 2 selected segments" {} \
		 -command Grh2::seg::intersect}
	}
	"&View" all tools 0 {
	    {command "Zoom &in" {} "Zoom in" {} \
		  -command zoomIn }
	    {command "Zoom &out" {} "Zoom out" {} \
		  -command zoomOut }
	}
	"&Help" all help 0 { 
	    {command "&About" {} "About Grh2" {} -command Grh2::aboutBox}
	    {command "How to start" {} "Startup" {} -command Grh2::howToStart}
	}
    }
    set ::Grh2::mainframe [ MainFrame .mainframe \
		       -menu $descmenu \
		       -textvariable Grh2::status \
		       -progressvar Grh2::progress]
    # --
    # -- Toolbar
    # --
    set Grh2::toolbar [$Grh2::mainframe addtoolbar]

    # -- Make bbox MAP FILE
    set bbox [ButtonBox $Grh2::toolbar.bbox1 -spacing 4 -padx 1 -pady 1]

    # Load map
#    $bbox add -image [Bitmap::get open] \
        -highlightthickness 0 -takefocus 0 -relief raised -borderwidth 1 -padx 1 -pady 1 \
        -helptext "Load map" -command Grh2::seg::loadMap

    # Save map
#    $bbox add -image [Bitmap::get save] \
        -highlightthickness 0 -takefocus 0 -relief raised -borderwidth 1 -padx 1 -pady 1 \
        -helptext "Save current map" -command Grh2::seg::saveMap

    # Pack and sep
#    pack $bbox -side left -anchor w
#    set sep [Separator $Grh2::toolbar.sep1 -orient vertical -relief ridge ]
#    pack $sep -side left -fill y -padx 5 -anchor w
    
    # -- Make bbox EDIT
    set bbox [ButtonBox $Grh2::toolbar.bbox3 -spacing 4 -padx 1 -pady 1]

    # add: delete segs
    $bbox add -image [Bitmap::get $::Grh2::dir/icons/delete] \
        -highlightthickness 0 -takefocus 0 -relief raised -borderwidth 1 -padx 1 -pady 1 \
	-helptext "Delete all selected segments" -command Grh2::seg::deleteSelected

    # add: delete segs in rectangle
    $bbox add -image [Bitmap::get $::Grh2::dir/icons/deleteIn] \
        -highlightthickness 0 -takefocus 0 -relief raised -borderwidth 1 -padx 1 -pady 1 \
	-helptext "Delete segments inside rectangle (made with the mouse and pressing <SHIFT>-<BUTTON1>)" -command "Grh2::seg::clip SEGLOC_REMOVE_IN"

    # add: delete segs out rectangle
    $bbox add -image [Bitmap::get $::Grh2::dir/icons/deleteOut] \
        -highlightthickness 0 -takefocus 0 -relief raised -borderwidth 1 -padx 1 -pady 1 \
	-helptext "Clip map: Delete segments outside rectangle  (made with the mouse and pressing <SHIFT>-<BUTTON1>)" -command "Grh2::seg::clip SEGLOC_REMOVE_OUT"

    # Change map frame
    $bbox add -image [Bitmap::get $::Grh2::dir/icons/frame] \
        -highlightthickness 0 -takefocus 0 -relief raised -borderwidth 1 -padx 1 -pady 1 \
	-helptext "Change map frame (with the mouse place the robot on the new origin)" -command " Grh2::seg::changeFrame "


    # add merge
#    $bbox add -image [Bitmap::get $::Grh2::dir/icons/merge] \
        -highlightthickness 0 -takefocus 0 -relief raised -borderwidth 1 -padx 1 -pady 1 \
	-helptext "Merge 2 segments" -command Grh2::seg::merge
#    pack $bbox -side left -anchor w

    # add intersect
#    $bbox add -image [Bitmap::get $::Grh2::dir/icons/intersect] \
        -highlightthickness 0 -takefocus 0 -relief raised -borderwidth 1 -padx 1 -pady 1 \
	-helptext "Intersect 2 segments" -command Grh2::seg::intersect

    pack $bbox -side left -anchor w

    set sep [Separator $Grh2::toolbar.sep2 -orient vertical -relief ridge ]
    pack $sep -side left -fill y -padx 10 -anchor w

    # Loc the robot
    set bbox [ButtonBox $Grh2::toolbar.bbox6 -spacing 4 -padx 1 -pady 1]
    $bbox add -image [Bitmap::get $::Grh2::dir/icons/robot?] \
        -highlightthickness 0 -takefocus 0 -relief raised -borderwidth 1 -padx 1 -pady 1 \
	-helptext "Localize (first place approximately the robot with the mouse, turn it with <CTRL>-mouse)" -command " Grh2::seg::localize "

    # Add sick data
    $bbox add -image [Bitmap::get $::Grh2::dir/icons/sick+] \
        -highlightthickness 0 -takefocus 0 -relief raised -borderwidth 1 -padx 1 -pady 1 \
	-helptext "Add segs in map (robot must be localized first)" -command " Grh2::seg::addSegs "

    pack $bbox -side left -anchor w

    set sep [Separator $Grh2::toolbar.sep3 -orient vertical -relief ridge ]
    pack $sep -side left -fill y -padx 10 -anchor w

    # Undo (reload previous map)
    set bbox [ButtonBox $Grh2::toolbar.bbox8 -spacing 4 -padx 1 -pady 1]
    $bbox add -image [Bitmap::get $::Grh2::dir/icons/undo] \
        -highlightthickness 0 -takefocus 0 -relief raised -borderwidth 1 -padx 1 -pady 1 \
	-helptext "undo: reload previous map" -command " Grh2::seg::undo "

    # Redo (reload next map)
    $bbox add -image [Bitmap::get $::Grh2::dir/icons/redo] \
        -highlightthickness 0 -takefocus 0 -relief raised -borderwidth 1 -padx 1 -pady 1 \
	-helptext "redo: reload next map" -command " Grh2::seg::redo "

    # Pack and sep
    pack $bbox -side left -anchor w

    # -- Make bbox ZOOM
    set bbox [ButtonBox $Grh2::toolbar.bbox2 -spacing 4 -padx 1 -pady 1]

    # add Zooms
    $bbox add -image [Bitmap::get $::Grh2::dir/icons/zoomIn] \
        -highlightthickness 0 -takefocus 0 -relief raised -borderwidth 1 -padx 1 -pady 1 \
	-helptext "Zoom in" -command zoomIn
    $bbox add -image [Bitmap::get $::Grh2::dir/icons/zoomOut] \
        -highlightthickness 0 -takefocus 0 -relief raised -borderwidth 1 -padx 1 -pady 1 \
	-helptext "Zoom out" -command zoomOut

    # Pack and sep
    pack $bbox -anchor e  -fill both -expand yes

    # --
    # -- Canvas
    # --
    set frame [$Grh2::mainframe getframe]

    # $frame configure  -class ScrolledCanvas 
    set ::Grh2::c [scrolledCanvas $frame]


    # --
    # -- Mouse and robot positions
    # --
    set posf [frame $Grh2::mainframe.posframe]

    set mpos [ label $posf.mpos -textvariable Grh2::seg::mousePosition ]
    set rpos [ label $posf.rpos -textvariable Grh2::seg::robotPosition ] 
    pack $mpos -side left -anchor w
    pack $rpos -side right -anchor e

    pack $posf -side top -fill both -expand no

    # --
    # -- Some results
    # --

    set rqstf [frame $Grh2::mainframe.rqstframe]

    set rqstl [ label $rqstf.rqst -textvariable Grh2::seg::rqst ]
    set nbsegsl [ label $rqstf.nbsegs -textvariable Grh2::seg::nbsegs ]

    pack $rqstl -side left -anchor w
    pack $nbsegsl -side right -anchor e

    pack $rqstf -side top -fill both -expand no


    # --
    # -- Main frame
    # --

    pack $Grh2::mainframe -fill both -expand yes

    $Grh2::mainframe addindicator -text " (c) LAAS/CNRS 1999-2007 " 
}

###----------------------------------------------------------------------
##
## Zoom
##
# 
# In
# 
proc zoomIn {} {
    ::Grh2::zoominProc 320 200
    Grh2::updateScrollBars
}

#
# Out
#
proc zoomOut {} {
    ::Grh2::zoomoutProc 320 200
}


#
# 
proc showPos {c x y} {
    set cx [$Grh2::c canvasx $x]
    set cy [$Grh2::c canvasy $y]

    set xm [Grh2::x_pixel_to_m $cx]
    set ym [Grh2::y_pixel_to_m $cy]
    
    $Grh2::mainframe configure -textvariable  "$xm $ym"
}


###----------------------------------------------------------------------
##
## Segment select/unselect
##
#
# Bindings and cursor for select mode
#
proc selectMode {} {
    $Grh2::c configure -cursor "left_ptr"

    $Grh2::c bind map <Enter> "hilightItem $Grh2::c"
    $Grh2::c bind map <ButtonPress-1> "selectItem $Grh2::c"
    $Grh2::c bind map <Leave> "restoreItem $Grh2::c"

#    $Grh2::c bind robot <Enter> "hilightItem $Grh2::c"
    $Grh2::c bind robot <ButtonPress-1> "Grh2::robsim::stopLoop"
#    $Grh2::c bind robot <Leave> "restoreItem $Grh2::c"
    $Grh2::c bind robot <B1-Motion> "Grh2::robsim::move $Grh2::c %x %y"
    $Grh2::c bind robot <Control-B1-Motion> "Grh2::robsim::rotate $Grh2::c %x %y"
    $Grh2::c bind robot <ButtonRelease-1> "Grh2::robsim::updateSegs $Grh2::c"

    bind $Grh2::c <Shift-Button-1> "initRectangle $Grh2::c %x %y"
    bind $Grh2::c <Shift-B1-Motion> "createRectangle $Grh2::c %x %y"
    bind $Grh2::c <Shift-ButtonRelease-1> "endRectangle $Grh2::c %x %y"
}

#
# "hover" mode: highlight
#
proc hilightItem {c} {
    global stateColor

    set id [$c find withtag current]
    
    $c itemconfigure $id -fill $stateColor(current)
}

#
# "hover" mode: restore original state
#
proc restoreItem {c} {
    global stateColor itemState
    
    set id [$c find withtag current]
    
    $c itemconfigure $id -fill $stateColor($itemState($id))
}

#
# select segment under the pointer
#   
proc selectItem {c} {
    global stateColor itemState selected

    set id [$c find withtag current]

    if  {! [info exists selected($id)]} {
	set itemState($id) selected
	$c itemconfigure $id -fill $stateColor(selected)
	set selected($id) true
    } else {
	set itemState($id) normal
	unset selected($id)
    }
}

#
# 
#
proc initRectangle {c x y} {
    set Grh2::seg::xrect1 $x
    set Grh2::seg::yrect1 $y
}

proc endRectangle {c x y} {
    set Grh2::seg::xrect2 $x
    set Grh2::seg::yrect2 $y
}

proc createRectangle {c x y} {
    set Grh2::seg::xrect2 $x
    set Grh2::seg::yrect2 $y

    $Grh2::c delete rect

    if { $Grh2::seg::xrect2 >= $Grh2::seg::xrect1 } {
	set x1 $Grh2::seg::xrect1
	set x2 $Grh2::seg::xrect2
    }  else {
	set x1 $Grh2::seg::xrect2
	set x2 $Grh2::seg::xrect1
    }

    if { $Grh2::seg::yrect2 >= $Grh2::seg::yrect1 } {
	set y1 $Grh2::seg::yrect1
	set y2 $Grh2::seg::yrect2
    }    else {
	set y1 $Grh2::seg::yrect2
	set y2 $Grh2::seg::yrect1
    }


    set x1 [$Grh2::c canvasx $x1]
    set y1 [$Grh2::c canvasy $y1]
    set x2 [$Grh2::c canvasx $x2]
    set y2 [$Grh2::c canvasy $y2]

    $Grh2::c create rectangle $x1 $y1 $x2 $y2 -outline blue -width 2 -tags rect
}

#----------------------------------------------------------------------
#
# Change the cursor to a watch during execution of 'code'
#
proc waitCursor {code} {
    $Grh2::c configure -cursor "watch"
    # XXX 
    uplevel $code				
    $Grh2::c configure -cursor "left_ptr"
}

#----------------------------------------------------------------------
##
## Re-read map from poster
## 
proc Grh2::seg::refreshMap {c} {
    global selected itemState stateColor

    # Remove all selection
    catch {unset selected} {} 
    catch {unset itemState} {} 

    # read map
    set nbsegs  [::seglocPoster::updateMap  -fill $stateColor(normal)]
    set Grh2::seg::nbsegs [concat  $nbsegs "segs"]
    
    # initial state for all segments
    foreach id [$c find withtag map] {
	set itemState($id) normal
    }
    return $nbsegs
}

#----------------------------------------------------------------------
##
## Re-read segs and pos from poster
## 
proc Grh2::seg::refreshSegs { } {
    global  stateColor

    # read segs
    set robPos [ ::seglocPoster::updateSegs $Grh2::seg::posByPosterFlag  $Grh2::seg::robX $Grh2::seg::robY $Grh2::seg::robAngle]

    # record robot pos from poster
    foreach {Grh2::seg::robX Grh2::seg::robY Grh2::seg::robAngle} $robPos { }

    # and do it again : loop every 500 ms
    if { $Grh2::seg::loopFlag } {
	after 500 Grh2::seg::refreshSegs
    }
}

#----------------------------------------------------------------------
##
## send a generic request
##    
proc Grh2::seg:sendRequest { rqst { params "" } } {

    # send request
    puts "rqst: $rqst params: $params"
    set retCode [catch {
	eval [concat segloc::$rqst -raw $params]
    } answer ]
    set message [lindex $answer 0]
    set Grh2::seg::rqst "Request $rqst: $message"

    # Init required !
    if { $message == "S_segloc_stdGenoM_WAIT_INIT_RQST" } {
	Grh2::seg::initModule
    }
    return $answer
}
#----------------------------------------------------------------------
##
## Init the module segloc with default params
##    
proc Grh2::seg::initModule {} {

    set file [Grh2::seg::askMapFile]
    if { $file == "" } {
	return "Need map file ! (to make a new map, chose file oneseg.map)"
    }
    waitCursor {
	set answer [Grh2::seg:sendRequest Init "$file -1 0 0 sickScanPolar"]
	set message [lindex $answer 0]
	if { $message == "OK" } {
	    Grh2::seg::getCurrentFileName
	    Grh2::seg::refreshMap $Grh2::c
	    Grh2::updateScrollBars
	}
    }
}


#----------------------------------------------------------------------
##
## Add segs in map
## 
proc Grh2::seg::addSegs { } {
    set answer [Grh2::seg:sendRequest AddSegsInMap ""]
    if { [lindex $answer 0] != "OK" } {
	return
    }

    Grh2::seg::saveMap
    Grh2::seg::refreshMap $::Grh2::c
}

#----------------------------------------------------------------------
##
## Change map frame
## 
proc Grh2::seg::changeFrame { } {
    set answer [Grh2::seg:sendRequest ResetFrame "$Grh2::seg::robX $Grh2::seg::robY  $Grh2::seg::robAngle 0 0 0"]
    if { [lindex $answer 0] != "OK" } {
	return
    }

    Grh2::seg::saveMap
    Grh2::seg::refreshMap $::Grh2::c
}

#----------------------------------------------------------------------
##
## Localize the robot
## 
proc Grh2::seg::localize { } {
    set answer [Grh2::seg:sendRequest LocLocal "$Grh2::seg::robX $Grh2::seg::robY $Grh2::seg::robAngle 2 2 1"]
    if { [lindex $answer 0] != "OK" } {
	puts "Loc failed. Place approximately the robot with the mouse (turn with <CNTRL>-Mouse) before trying again."
	return
    }

    set answer [Grh2::seg:sendRequest LocLoop -ack]
# XXXXXX
#    if { [lindex $answer 0] != "OK" } {
#	return
#    }

    set Grh2::seg::posByPosterFlag 1
    set Grh2::seg::loopFlag 1
    Grh2::seg::refreshSegs
}


#----------------------------------------------------------------------
## 
## Get the current map file name from segloc
##
proc Grh2::seg::getCurrentFileName {} {
    set answer [Grh2::seg:sendRequest GetMapFile ]
    if { [lindex $answer 0] != "OK" } {
	return
    }
    set ::Grh2::seg::currentFileName [lindex $answer 1]
    set ::Grh2::seg::currentFileNum  [lindex $answer 2]
    if { $::Grh2::seg::currentFileNum >= 0 } {
	puts "current file : $::Grh2::seg::currentFileName.$::Grh2::seg::currentFileNum"
	wm title . "seglocEdit - $::Grh2::seg::currentFileName.$::Grh2::seg::currentFileNum"
    } else {
	puts "current file : $::Grh2::seg::currentFileName"
	wm title . "seglocEdit - $::Grh2::seg::currentFileName"
	set Grh2::seg::currentFileNum 0
    }
}

#----------------------------------------------------------------------
##
## Ask for map file
##    
proc Grh2::seg::askMapFile {} {

    set types {
	{{All files} * }
    }
    set dir [file dirname $::Grh2::seg::currentFileName]

    set file [tk_getOpenFile -filetypes $types -initialdir $dir \
		  -title "select map file..."]
    return $file
}

#----------------------------------------------------------------------
##
## Load a new map
##    
proc Grh2::seg::loadMapRqst { file } {
    if { $file == "" } {
	puts "no file"
	return
    }
    waitCursor {
	set answer [Grh2::seg:sendRequest LoadMap "$file"]
	if { [lindex $answer 0] == "OK" } {
	    Grh2::seg::getCurrentFileName
	    Grh2::seg::refreshMap $Grh2::c
	    Grh2::updateScrollBars
	}
    }
}
#----------------------------------------------------------------------
##
## Load a new map from file
##    
proc Grh2::seg::loadMap {} {
    Grh2::seg::loadMapRqst [concat [Grh2::seg::askMapFile] "-1 0"]
}

#----------------------------------------------------------------------
##
## Load the previous file map
##    
proc Grh2::seg::undo {} {

    # get previous file
    set ::Grh2::seg::currentFileNum [expr $::Grh2::seg::currentFileNum - 1]
    if { $::Grh2::seg::currentFileNum < -1 } {
	set ::Grh2::seg::currentFileNum -1
	puts "No previous map"
	return
    }

    Grh2::seg::loadMapRqst "$::Grh2::seg::currentFileName $::Grh2::seg::currentFileNum 0"
}

#----------------------------------------------------------------------
##
## Load the next file map
##    
proc Grh2::seg::redo {} {

    # get next file
    set ::Grh2::seg::currentFileNum [expr $::Grh2::seg::currentFileNum + 1]

    Grh2::seg::loadMapRqst "$::Grh2::seg::currentFileName $::Grh2::seg::currentFileNum 0"
}

#----------------------------------------------------------------------
##
## Save current map in current file
##
proc Grh2::seg::saveMap {} {
    
    set ::Grh2::seg::currentFileNum [expr $::Grh2::seg::currentFileNum + 1]
    waitCursor {
	set answer [Grh2::seg:sendRequest SaveMap "$::Grh2::seg::currentFileName $::Grh2::seg::currentFileNum 0"]
	if { [lindex $answer 0] == "OK" } {
	    Grh2::seg::getCurrentFileName
	}
    }
}

#----------------------------------------------------------------------
##
## Save current map in a new file
##
proc Grh2::seg::saveMapAs {} {

    set types {
	{{Map files} {.map}}
	{{All files} * }
    }
    set dir [file dirname $::Grh2::seg::currentFileName]
    set file [tk_getSaveFile -filetypes $types -initialdir $dir \
		  -defaultextension ".map" -title "save map as..."]
    if {$file != ""} {
	set answer [Grh2::seg:sendRequest SaveMap "$file -1 0"]
	if { [lindex $answer 0] != "OK" } {
	    return
	}
	Grh2::seg::getCurrentFileName
    }
}

#----------------------------------------------------------------------
###
### map editing functions
###
#
# Delete all selectionned segments
#
proc Grh2::seg::deleteSelected {} {
    global selected 
    set answer 0

    waitCursor {
	foreach id [ array names selected ] {
	    set answer [Grh2::seg:sendRequest RemoveSegment "$::Grh2::seg::segsCoord($id)"]
	    if { [lindex $answer 0] != "OK" } {
		break
	    }
	    Grh2::seg::refreshMap $::Grh2::c
	}
	if { [lindex $answer 0] == "OK" } {
	    Grh2::seg::saveMap
	    Grh2::seg::refreshMap $Grh2::c
	    Grh2::updateScrollBars
	}
    }
}

#
# Delete all segments inside or outside the rectangle
#
proc Grh2::seg::clip { clipMode } {

    if { $Grh2::seg::xrect1 == 0 && $Grh2::seg::xrect2 == 0 } {
	puts "First make a rectangle pressing <SHIFT>-<BUTTON1> and moving the mouse"
	set Grh2::seg::rqst "First make a rectangle pressing <SHIFT>-<BUTTON1> and moving the mouse"
	return
    }

    set cx1 [$Grh2::c canvasx $Grh2::seg::xrect1]
    set cy1 [$Grh2::c canvasy $Grh2::seg::yrect1]
    set cx2 [$Grh2::c canvasx $Grh2::seg::xrect2]
    set cy2 [$Grh2::c canvasy $Grh2::seg::yrect2]

    set x1 [Grh2::x_pixel_to_m $cx1]
    set y1 [Grh2::y_pixel_to_m $cy1]
    set x2 [Grh2::x_pixel_to_m $cx2]
    set y2 [Grh2::y_pixel_to_m $cy2]
    
    if { $x1 > $x2 } {
	set x  $x1
	set x1 $x2
	set x2 $x
    }
    if { $y1 > $y2 } {
	set y  $y1
	set y1 $y2
	set y2 $y
    }

    waitCursor {
	set answer [Grh2::seg:sendRequest ClipMap "$clipMode 0 $x1 $y1 $x2 $y2"]
	if { [lindex $answer 0] == "OK" } {
	    Grh2::seg::saveMap
	    Grh2::seg::refreshMap $Grh2::c
	    Grh2::updateScrollBars
	}
    }
    $Grh2::c delete rect
}

#----------------------------------------------------------------------
#
# Merge 2 colinear segments
#
proc Grh2::seg::merge {} {
    global selected

    set sl [array names selected]
    if {[llength $sl] == 2} {
	set s1 $::Grh2::seg::segsCoord([lindex $sl 0])
	set s2 $::Grh2::seg::segsCoord([lindex $sl 1])
	waitCursor {
	    set answer [Grh2::seg:sendRequest MergeSegments "$s1 $s2"]
	    if { [lindex $answer 0] == "OK" } {
		Grh2::seg::refreshMap $::Grh2::c
	    }
	}
    }
}

#----------------------------------------------------------------------
#
# Recompute vertices at the intersection of 2 segments
#
proc Grh2::seg::intersect {} {
    global selected

    set sl [array names selected]
    if {[llength $sl] == 2} {
	set s1 $::Grh2::seg::segsCoord([lindex $sl 0])
	set s2 $::Grh2::seg::segsCoord([lindex $sl 1])
	waitCursor {
	    set answer [Grh2::seg:sendRequest IntersectSegments "$s1 $s2"]
	    if { [lindex $answer 0] == "OK" } {
		Grh2::seg::refreshMap $::Grh2::c
	    }
	}
    }
}

#----------------------------------------------------------------------
#
# show permanently the current position
#
proc showPos {c x y} {
    set cx [$Grh2::c canvasx $x]
    set cy [$Grh2::c canvasy $y]

    set x [Grh2::x_pixel_to_m $cx]
    set y [Grh2::y_pixel_to_m $cy]
    
    set Grh2::seg::mousePosition [format "x %.2f y %.2f" $x $y]
    set Grh2::seg::robotPosition [format "robot: %.2f m %.2f m %.3f rd" $Grh2::seg::robX $Grh2::seg::robY  $Grh2::seg::robAngle]
}


#----------------------------------------------------------------------

#
# Marketing
#
proc Grh2::aboutBox {} {
    tk_messageBox -icon info -type ok -title "About seglocEdit" \
	-message  "Grh2 - seglocEdit\n\nCopyright (c) 1999-2007 LAAS/CNRS"
}

# Help start
proc Grh2::howToStart {} {
    tk_messageBox -icon info -type ok -title "How to use seglocEdit" \
	-message "See file README-seglocEdit in <openrobots>/grh2/tcl"
}

#----------------------------------------------------------------------

#
# MOVE THE ROBOT WITH THE MOUSE
# 
namespace eval ::Grh2::robsim { 
    proc stopLoop { } {
	set Grh2::seg::loopFlag 0
	set Grh2::seg::posByPosterFlag 0
    }
    proc move {c x y} {
	set Grh2::seg::loopFlag 0
	set Grh2::seg::posByPosterFlag 0
	set id [$c find withtag current]
	set cx [$c canvasx $x]
	set cy [$c canvasy $y]
	set old [$c coords $id]
	set Grh2::seg::robX    [Grh2::x_pixel_to_m $cx]
	set Grh2::seg::robY    [Grh2::y_pixel_to_m $cy]

	$c coords $id $cx $cy [lindex $old 2]

    }
    proc rotate {c x y} {
	set Grh2::seg::loopFlag 0
	set Grh2::seg::posByPosterFlag 0
	set id [$c find withtag current]
	set x [$c canvasx $x]
	set y [$c canvasy $y]
	set center [$c coords $id]
	set cx [lindex $center 0]
	set cy [lindex $center 1]
	set cangle [lindex $center 2]
	set Grh2::seg::robAngle [expr 3.1415926535 - atan2($cy - $y, $cx - $x)]
	$c coords $id $cx $cy $Grh2::seg::robAngle

    }
    proc updateSegs {c} {
	set Grh2::seg::posByPosterFlag 0
	set Grh2::seg::loopFlag 0
	Grh2::seg::refreshSegs
#	::seglocPoster::updateSegs $Grh2::seg::posByPosterFlag $Grh2::seg::robX $Grh2::seg::robY $Grh2::seg::robAngle
    }
}

#----------------------------------------------------------------------

##
## Go 
##


# Required packages
package require Grh2
package require genom

# load resources
Grh2::getResources $::Grh2::dir/app-defaults/Graphique
# Creation fenetre 
Grh2::mainWindow
wm title . "Grh2 - seglocEdit"
wm deiconify .

# define an icon
toplevel .icon
image create photo icon 
icon read $::Grh2::dir/icons/diligent.gif
label .icon.l -image icon
pack .icon.l
wm iconwindow . .icon
wm iconname . editSegloc

# Interface with seglog posters
package require Grh2::seglocPoster

# default mode: select segments
selectMode
# Connect to tclserv
if { [ catch { connect localhost} result ] } {
    puts $result
    puts "Could not connect to tclserv on localhost. Start it first"
    exit 1
}
if { [ catch { lm segloc } result ] } {
    puts $result
    puts "Could not load module segloc. Install it first" 
    exit 1
}
if { [catch { mboxInit } result ] } { 
    puts $result
    puts "Could not find module segloc. Start it first"
    exit 1
}

Grh2::seg::getCurrentFileName
waitCursor {Grh2::seg::refreshMap $::Grh2::c}
Grh2::updateScrollBars

Grh2::seg::refreshSegs

bind $Grh2::c <Motion> {showPos $Grh2::c %x %y}
#bind $Grh2::c <Any-ButtonRelease-1> "arrowSetup $c"

bind $Grh2::c <ButtonPress-1> {showPos $Grh2::c %x %y}
