#
# Copyright (c) 2000-2006 CNRS/LAAS
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
# $LAAS: seglocPoster.tcl,v 1.9 2007/07/23 09:22:40 sara Exp $
#
package require http
package require tdom

namespace eval seglocPoster {

    proc init {} {
    }

    proc getDoc { poster } {

	set url "http://localhost:8080/segloc?get=$poster"
	set token [http::geturl "$url"]
	if {[http::status $token] != "ok"} {
	    http::cleanup $token
	    error "bad status [http::status $token]"
	}
	set xml [http::data $token]
	set dom [dom parse $xml]
	return [$dom documentElement]
    }

    proc updateMap { {args {}} } {
	set doc [getDoc MapMap]
	set data [$doc selectNodes /segloc/MapMap/data]
	set nbSegs [[$data selectNodes header/nbSegs/text()] data]
	puts "$nbSegs map segs"
	$::Grh2::c delete map
	for { set i 0 } { $i < $nbSegs } { incr i } {
	    set seg [$data selectNodes segs.$i]
	    set x1 [[$seg selectNodes x1/text()] data]
	    set y1 [[$seg selectNodes y1/text()] data]
	    set x2 [[$seg selectNodes x2/text()] data]
	    set y2 [[$seg selectNodes y2/text()] data]
	    set id [$::Grh2::c create line \
		[::Grh2::x_m_to_pixel $x1] \
		[::Grh2::y_m_to_pixel $y1] \
		[::Grh2::x_m_to_pixel $x2] \
		[::Grh2::y_m_to_pixel $y2] \
			-tags map]
	    set ::Grh2::seg::segsCoord($id) "$x1 $y1 $x2 $y2"
	}
	return $nbSegs
    }

    # LOC BY SEGLOC
    proc ReadSeglocLoc { {args {}} } {
	global xt yt ct st tt colorLoc

	set doc [getDoc Loc]
	set data [$doc selectNodes /segloc/Loc/data]
	set xt [[$data selectNodes Transf/pos/x/text()] data]
	set yt [[$data selectNodes Transf/pos/y/text()] data]
	set tt [[$data selectNodes Transf/pos/theta/text()] data]
	set quality [[$data selectNodes Report/quality/text()] data]
	set ct [expr cos($tt)]
	set st [expr sin($tt)]
#	puts "xt $xt yt $yt tt $tt  quality $quality"
	if { $quality == "SEGLOC_LOC_QUALITY_OK"} {
	    set colorLoc green
	} else {
	    set colorLoc red
	}	    
    }

    # SEGMENTATION BY SEGLOC
    proc updateSegs { posPosterFlag {x 0} {y 0} {th 0} } {
	global xt yt ct st tt colorLoc
	global data nbSegs

#	puts "nbSegs = $nbSegs"
#	set xSickOnRob [[$data selectNodes header/sickPomPos/sensorToMain/euler/x/text()] data]
#	set ySickOnRob [[$data selectNodes header/sickPomPos/sensorToMain/euler/y/text()] data]
#	set thSickOnRob [[$data selectNodes header/sickPomPos/sensorToMain/euler/yaw/text()] data]

#	puts "sensorToMain: SickOnRob $xSickOnRob m $ySickOnRob m $thSickOnRob rd"
	if { $posPosterFlag == 1 } {
	    set doc [getDoc SegsSegs]
	    set data [$doc selectNodes /segloc/SegsSegs/data]
	    set nbSegs [[$data selectNodes header/nbSegs/text()] data]

	    ReadSeglocLoc

	    set xOdo [[$data selectNodes header/sickPomPos/mainToOrigin/euler/x/text()] data]
	    set yOdo [[$data selectNodes header/sickPomPos/mainToOrigin/euler/y/text()] data]
	    set thOdo [[$data selectNodes header/sickPomPos/mainToOrigin/euler/yaw/text()] data]

#	    puts "mainToOrigin: xOdo $xOdo yOdo $yOdo tOdo $thOdo"

	    set xRob  [ expr $xt  +  $ct * $xOdo  -  $st * $yOdo ]
	    set yRob  [ expr $yt  +  $st * $xOdo  +  $ct * $yOdo ]
	    set thRob [ expr $tt  +  $thOdo ]
	    set cosRob [expr cos($thRob) ]
	    set sinRob [expr sin($thRob) ]

	    set colorSegs $colorLoc

	} else {
	    set xRob $x
	    set yRob $y
	    set thRob $th
	    set cosRob [expr cos($thRob) ]
	    set sinRob [expr sin($thRob) ]

	    set colorSegs blue
	}

#	puts "nbSegs $nbSegs xRob $xRob yRob $yRob tRob $thRob"

	$Grh2::c delete robot
	set x [Grh2::x_m_to_pixel $xRob ]
	set y [Grh2::y_m_to_pixel $yRob ]
	set theta $thRob
	$Grh2::c create robot $x $y $theta -type xr4000 -fill $colorSegs -tag robot

	$::Grh2::c delete segs

	for { set i 0 } { $i < $nbSegs } { incr i } {
	    set seg [$data selectNodes segs.$i]
	    set x1 [[$seg selectNodes x1/text()] data]
	    set y1 [[$seg selectNodes y1/text()] data]
	    set x2 [[$seg selectNodes x2/text()] data]
	    set y2 [[$seg selectNodes y2/text()] data]

	    set xg1 [ expr $xRob + $x1 * $cosRob - $y1 * $sinRob ]
	    set yg1 [ expr $yRob + $y1 * $cosRob + $x1 * $sinRob ]

	    set xg2 [ expr $xRob + $x2 * $cosRob - $y2 * $sinRob ]
	    set yg2 [ expr $yRob + $y2 * $cosRob + $x2 * $sinRob ]

	    set id [$::Grh2::c create line \
		[::Grh2::x_m_to_pixel $xg1] \
		[::Grh2::y_m_to_pixel $yg1] \
		[::Grh2::x_m_to_pixel $xg2] \
		[::Grh2::y_m_to_pixel $yg2] \
			-tags segs -fill $colorSegs]
	}
	return "$xRob $yRob $thRob"
    }
}

package provide Grh2::seglocPoster 1.0
