#
# Copyright (c) 1999 LAAS/CNRS 
#
# Matthieu Herrb - Tue Sep 14 1999
#
# $Id: tk_entrydialog.tcl,v 1.2 2006/03/17 16:01:19 matthieu Exp $
#
package provide Grh2::entrydialog 1.0

proc tk_entrydialog {w text} {
    global tkPriv tcl_platform

    # 1. Create the top-level window and divide it into top
    # and bottom parts.

    catch {destroy $w}
    toplevel $w -class Dialog
    wm iconname $w Dialog
    wm title $w {}
    wm protocol $w WM_DELETE_WINDOW {} 

    # The following command means that the dialog won't be posted if
    # [winfo parent $w] is iconified, but it's really needed;  otherwise
    # the dialog can become obscured by other windows in the application,
    # even though its grab keeps the rest of the application from being used.
 
    wm transient $w [winfo toplevel [winfo parent $w]]

    frame $w.bot
    frame $w.top
    $w.bot configure -relief raised -bd 1
    $w.top configure -relief raised -bd 1
    pack $w.bot -side bottom -fill both
    pack $w.top -side top -fill both -expand 1
    
    label $w.msg -justify left -text $text
    pack $w.msg -in $w.top -side left -expand 1 -padx 3m -pady 3m
    
    entry $w.entry -width 20
    bind $w.entry <Return> [concat {set tkPriv(button) [} "$w.entry get" {]}]
    pack $w.entry -in $w.top -side right -expand 1 -padx 3m -pady 3m

    # 3. Create a row of buttons at the bottom of the dialog.
 
    set i 0
    button $w.ok -text "Ok" -command [concat {set tkPriv(button) [} \
						  "$w.entry get" {]}]
    grid $w.ok -in $w.bot -column 0 -row 0 -sticky ew -padx 10 -pady 10
    button $w.cancel -text "Cancel" -command "set tkPriv(button) -1"
    grid $w.cancel -in $w.bot -column 1 -row 0 -sticky ew -padx 10 -pady 10

    # 5. Create a <Destroy> binding for the window that sets the
    # button variable to -1;  this is needed in case something happens
    # that destroys the window, such as its parent window being destroyed.
 
    bind $w <Destroy> {set tkPriv(button) -1}
 
    # 6. Withdraw the window, then update all the geometry information
    # so we know how big it wants to be, then center the window in the
    # display and de-iconify it.
 
    wm withdraw $w
    update idletasks
    set x [expr {[winfo screenwidth $w]/2 - [winfo reqwidth $w]/2 \
            - [winfo vrootx [winfo parent $w]]}]
    set y [expr {[winfo screenheight $w]/2 - [winfo reqheight $w]/2 \
            - [winfo vrooty [winfo parent $w]]}]
    wm geom $w +$x+$y
    wm deiconify $w
 

    # 7. Set a grab and claim the focus too.
    
    set oldFocus [focus]
    set oldGrab [grab current $w]
    if {[string compare $oldGrab ""]} {
        set grabStatus [grab status $oldGrab]
    }
    grab $w
    focus $w.entry
    
    # 8. Wait for the user to respond, then restore the focus and
    # return the index of the selected button.  Restore the focus
    # before deleting the window, since otherwise the window manager
    # may take the focus away so we can't redirect it.  Finally,
    # restore any grab that was in effect.
 
    tkwait variable tkPriv(button)
    catch {focus $oldFocus}
    catch {
        # It's possible that the window has already been destroyed,
        # hence this "catch".  Delete the Destroy handler so that
        # tkPriv(button) doesn't get reset by it.
 
        bind $w <Destroy> {}
        destroy $w
    }
    if {[string compare $oldGrab ""]} {
      if {[string compare $grabStatus "global"]} {
            grab $oldGrab
      } else {
          grab -global $oldGrab
        }
    }
    return $tkPriv(button)
}
